@@ -17,7 +17,7 @@ fn main() {
         .unwrap();
 
     event_loop.run(move |event, _, control_flow| {
-        println!("{:?}", event);
+        println!("{event:?}");
 
         control_flow.set_wait();
 
