@@ -22,7 +22,7 @@ fn main() {
     let timer_length = Duration::new(1, 0);
 
     event_loop.run(move |event, _, control_flow| {
-        println!("{:?}", event);
+        println!("{event:?}");
 
         match event {
             Event::NewEvents(StartCause::Init) => {
