@@ -1,5 +1,6 @@
 use std::cell::Cell;
 use std::mem::ManuallyDrop;
+use std::sync::atomic::AtomicBool;
 use std::sync::{Arc, Mutex};
 
 use sctk::reexports::client::protocol::wl_compositor::WlCompositor;
@@ -10,6 +11,7 @@ use sctk::reexports::protocols::staging::xdg_activation::v1::client::xdg_activat
 
 use sctk::environment::Environment;
 use sctk::window::{Decorations, Window};
+use wayland_protocols::viewporter::client::wp_viewport::WpViewport;
 
 use crate::dpi::{LogicalPosition, LogicalSize};
 
@@ -17,10 +19,11 @@ use crate::event::{Ime, WindowEvent};
 use crate::platform_impl::wayland;
 use crate::platform_impl::wayland::env::WinitEnv;
 use crate::platform_impl::wayland::event_loop::{EventSink, WinitState};
+use crate::platform_impl::wayland::protocols::wp_fractional_scale_v1::WpFractionalScaleV1;
 use crate::platform_impl::wayland::seat::pointer::WinitPointer;
 use crate::platform_impl::wayland::seat::text_input::TextInputHandler;
 use crate::platform_impl::wayland::WindowId;
-use crate::window::{CursorGrabMode, CursorIcon, Theme, UserAttentionType};
+use crate::window::{CursorGrabMode, CursorIcon, ImePurpose, Theme, UserAttentionType};
 
 use super::WinitFrame;
 
@@ -59,9 +62,6 @@ pub enum WindowRequest {
     /// Request decorations change.
     Decorate(bool),
 
-    /// Request decorations change.
-    CsdThemeVariant(Theme),
-
     /// Make the window resizeable.
     Resizeable(bool),
 
@@ -83,6 +83,12 @@ pub enum WindowRequest {
     /// Enable IME on the given window.
     AllowIme(bool),
 
+    /// Set the IME purpose.
+    ImePurpose(ImePurpose),
+
+    /// Mark the window as opaque.
+    Transparent(bool),
+
     /// Request Attention.
     ///
     /// `None` unsets the attention request.
@@ -96,6 +102,9 @@ pub enum WindowRequest {
 
     /// Window should be closed.
     Close,
+
+    /// Change window theme.
+    Theme(Option<Theme>),
 }
 
 // The window update comming from the compositor.
@@ -105,7 +114,7 @@ pub struct WindowCompositorUpdate {
     pub size: Option<LogicalSize<u32>>,
 
     /// New scale factor.
-    pub scale_factor: Option<i32>,
+    pub scale_factor: Option<f64>,
 
     /// Close the window.
     pub close_window: bool,
@@ -139,6 +148,12 @@ pub struct WindowHandle {
     /// An actual window.
     pub window: ManuallyDrop<Window<WinitFrame>>,
 
+    /// The state of the fractional scaling handlers for the window.
+    pub fractional_scaling_state: Option<FractionalScalingState>,
+
+    /// The scale factor of the window.
+    pub scale_factor: Arc<Mutex<f64>>,
+
     /// The current size of the window.
     pub size: Arc<Mutex<LogicalSize<u32>>>,
 
@@ -151,9 +166,18 @@ pub struct WindowHandle {
     /// Whether the window is resizable.
     pub is_resizable: Cell<bool>,
 
+    /// Whether the window has keyboard focus.
+    pub has_focus: Arc<AtomicBool>,
+
     /// Allow IME events for that window.
     pub ime_allowed: Cell<bool>,
 
+    /// IME purpose for that window.
+    pub ime_purpose: Cell<ImePurpose>,
+
+    /// Wether the window is transparent.
+    pub transparent: Cell<bool>,
+
     /// Visible cursor or not.
     cursor_visible: Cell<bool>,
 
@@ -181,6 +205,9 @@ impl WindowHandle {
         env: &Environment<WinitEnv>,
         window: Window<WinitFrame>,
         size: Arc<Mutex<LogicalSize<u32>>>,
+        has_focus: Arc<AtomicBool>,
+        fractional_scaling_state: Option<FractionalScalingState>,
+        scale_factor: f64,
         pending_window_requests: Arc<Mutex<Vec<WindowRequest>>>,
     ) -> Self {
         let xdg_activation = env.get_global::<XdgActivationV1>();
@@ -190,10 +217,13 @@ impl WindowHandle {
 
         Self {
             window: ManuallyDrop::new(window),
+            fractional_scaling_state,
+            scale_factor: Arc::new(Mutex::new(scale_factor)),
             size,
             pending_window_requests,
             cursor_icon: Cell::new(CursorIcon::Default),
             is_resizable: Cell::new(true),
+            transparent: Cell::new(false),
             cursor_grab_mode: Cell::new(CursorGrabMode::None),
             cursor_visible: Cell::new(true),
             pointers: Vec::new(),
@@ -202,9 +232,15 @@ impl WindowHandle {
             attention_requested: Cell::new(false),
             compositor,
             ime_allowed: Cell::new(false),
+            ime_purpose: Cell::new(ImePurpose::default()),
+            has_focus,
         }
     }
 
+    pub fn scale_factor(&self) -> f64 {
+        *self.scale_factor.lock().unwrap()
+    }
+
     pub fn set_cursor_grab(&self, mode: CursorGrabMode) {
         // The new requested state matches the current confine status, return.
         let old_mode = self.cursor_grab_mode.replace(mode);
@@ -349,6 +385,19 @@ impl WindowHandle {
         }
     }
 
+    pub fn set_transparent(&self, transparent: bool) {
+        self.transparent.set(transparent);
+        let surface = self.window.surface();
+        if transparent {
+            surface.set_opaque_region(None);
+        } else {
+            let region = self.compositor.create_region();
+            region.add(0, 0, i32::MAX, i32::MAX);
+            surface.set_opaque_region(Some(&region.detach()));
+            region.destroy();
+        }
+    }
+
     pub fn set_ime_allowed(&self, allowed: bool, event_sink: &mut EventSink) {
         if self.ime_allowed.get() == allowed {
             return;
@@ -357,8 +406,9 @@ impl WindowHandle {
         self.ime_allowed.replace(allowed);
         let window_id = wayland::make_wid(self.window.surface());
 
+        let purpose = allowed.then(|| self.ime_purpose.get());
         for text_input in self.text_inputs.iter() {
-            text_input.set_input_allowed(allowed);
+            text_input.set_input_allowed(purpose);
         }
 
         let event = if allowed {
@@ -370,6 +420,20 @@ impl WindowHandle {
         event_sink.push_window_event(event, window_id);
     }
 
+    pub fn set_ime_purpose(&self, purpose: ImePurpose) {
+        if self.ime_purpose.get() == purpose {
+            return;
+        }
+
+        self.ime_purpose.replace(purpose);
+
+        if self.ime_allowed.get() {
+            for text_input in self.text_inputs.iter() {
+                text_input.set_content_type_by_purpose(purpose);
+            }
+        }
+    }
+
     pub fn set_cursor_visible(&self, visible: bool) {
         self.cursor_visible.replace(visible);
         let cursor_icon = match visible {
@@ -433,6 +497,9 @@ pub fn handle_window_requests(winit_state: &mut WinitState) {
                     let event_sink = &mut winit_state.event_sink;
                     window_handle.set_ime_allowed(allow, event_sink);
                 }
+                WindowRequest::ImePurpose(purpose) => {
+                    window_handle.set_ime_purpose(purpose);
+                }
                 WindowRequest::SetCursorGrabMode(mode) => {
                     window_handle.set_cursor_grab(mode);
                 }
@@ -452,6 +519,13 @@ pub fn handle_window_requests(winit_state: &mut WinitState) {
                 WindowRequest::Minimize => {
                     window_handle.window.set_minimized();
                 }
+                WindowRequest::Transparent(transparent) => {
+                    window_handle.set_transparent(transparent);
+
+                    // This requires surface commit.
+                    let window_request = window_user_requests.get_mut(window_id).unwrap();
+                    window_request.redraw_requested = true;
+                }
                 WindowRequest::Decorate(decorate) => {
                     let decorations = match decorate {
                         true => Decorations::FollowServer,
@@ -464,15 +538,6 @@ pub fn handle_window_requests(winit_state: &mut WinitState) {
                     let window_request = window_user_requests.get_mut(window_id).unwrap();
                     window_request.refresh_frame = true;
                 }
-                #[cfg(feature = "sctk-adwaita")]
-                WindowRequest::CsdThemeVariant(theme) => {
-                    window_handle.window.set_frame_config(theme.into());
-
-                    let window_requst = window_user_requests.get_mut(window_id).unwrap();
-                    window_requst.refresh_frame = true;
-                }
-                #[cfg(not(feature = "sctk-adwaita"))]
-                WindowRequest::CsdThemeVariant(_) => {}
                 WindowRequest::Resizeable(resizeable) => {
                     window_handle.window.set_resizable(resizeable);
 
@@ -537,6 +602,18 @@ pub fn handle_window_requests(winit_state: &mut WinitState) {
                     let event_sink = &mut winit_state.event_sink;
                     event_sink.push_window_event(WindowEvent::Destroyed, *window_id);
                 }
+                WindowRequest::Theme(_theme) => {
+                    #[cfg(feature = "sctk-adwaita")]
+                    {
+                        window_handle.window.set_frame_config(match _theme {
+                            Some(theme) => theme.into(),
+                            None => sctk_adwaita::FrameConfig::auto(),
+                        });
+
+                        let window_requst = window_user_requests.get_mut(window_id).unwrap();
+                        window_requst.refresh_frame = true;
+                    }
+                }
             };
         }
     }
@@ -551,6 +628,9 @@ pub fn handle_window_requests(winit_state: &mut WinitState) {
 
 impl Drop for WindowHandle {
     fn drop(&mut self) {
+        // Drop the fractional scaling before the surface.
+        let _ = self.fractional_scaling_state.take();
+
         unsafe {
             let surface = self.window.surface().clone();
             // The window must be destroyed before wl_surface.
@@ -559,3 +639,28 @@ impl Drop for WindowHandle {
         }
     }
 }
+
+/// Fractional scaling objects.
+pub struct FractionalScalingState {
+    /// The wp-viewport of the window.
+    pub viewport: WpViewport,
+
+    /// The wp-fractional-scale of the window surface.
+    pub fractional_scale: WpFractionalScaleV1,
+}
+
+impl FractionalScalingState {
+    pub fn new(viewport: WpViewport, fractional_scale: WpFractionalScaleV1) -> Self {
+        Self {
+            viewport,
+            fractional_scale,
+        }
+    }
+}
+
+impl Drop for FractionalScalingState {
+    fn drop(&mut self) {
+        self.viewport.destroy();
+        self.fractional_scale.destroy();
+    }
+}
