@@ -2,8 +2,6 @@ use std::{cell::RefCell, collections::HashMap, rc::Rc, slice, sync::Arc};
 
 use libc::{c_char, c_int, c_long, c_uint, c_ulong};
 
-use parking_lot::MutexGuard;
-
 use super::{
     events, ffi, get_xtarget, mkdid, mkwid, monitor, util, Device, DeviceId, DeviceInfo, Dnd,
     DndState, GenericEventCookie, ImeReceiver, ScrollOrientation, UnownedWindow, WindowId,
@@ -59,7 +57,7 @@ impl<T: 'static> EventProcessor<T> {
         F: Fn(&Arc<UnownedWindow>) -> Ret,
     {
         let mut deleted = false;
-        let window_id = WindowId(window_id as u64);
+        let window_id = WindowId(window_id as _);
         let wt = get_xtarget(&self.target);
         let result = wt
             .windows
@@ -349,11 +347,11 @@ impl<T: 'static> EventProcessor<T> {
 
                     // These are both in physical space.
                     let new_inner_size = (xev.width as u32, xev.height as u32);
-                    let new_inner_position = (xev.x as i32, xev.y as i32);
-
-                    let mut shared_state_lock = window.shared_state.lock();
+                    let new_inner_position = (xev.x, xev.y);
 
                     let (mut resized, moved) = {
+                        let mut shared_state_lock = window.shared_state_lock();
+
                         let resized =
                             util::maybe_change(&mut shared_state_lock.size, new_inner_size);
                         let moved = if is_synthetic {
@@ -380,7 +378,13 @@ impl<T: 'static> EventProcessor<T> {
                         (resized, moved)
                     };
 
-                    let new_outer_position = if moved || shared_state_lock.position.is_none() {
+                    let position = window.shared_state_lock().position;
+
+                    let new_outer_position = if let (Some(position), false) = (position, moved) {
+                        position
+                    } else {
+                        let mut shared_state_lock = window.shared_state_lock();
+
                         // We need to convert client area position to window position.
                         let frame_extents = shared_state_lock
                             .frame_extents
@@ -395,21 +399,21 @@ impl<T: 'static> EventProcessor<T> {
                         let outer = frame_extents
                             .inner_pos_to_outer(new_inner_position.0, new_inner_position.1);
                         shared_state_lock.position = Some(outer);
+
+                        // Unlock shared state to prevent deadlock in callback below
+                        drop(shared_state_lock);
+
                         if moved {
-                            // Temporarily unlock shared state to prevent deadlock
-                            MutexGuard::unlocked(&mut shared_state_lock, || {
-                                callback(Event::WindowEvent {
-                                    window_id,
-                                    event: WindowEvent::Moved(outer.into()),
-                                });
+                            callback(Event::WindowEvent {
+                                window_id,
+                                event: WindowEvent::Moved(outer.into()),
                             });
                         }
                         outer
-                    } else {
-                        shared_state_lock.position.unwrap()
                     };
 
                     if is_synthetic {
+                        let mut shared_state_lock = window.shared_state_lock();
                         // If we don't use the existing adjusted value when available, then the user can screw up the
                         // resizing by dragging across monitors *without* dropping the window.
                         let (width, height) = shared_state_lock
@@ -441,15 +445,15 @@ impl<T: 'static> EventProcessor<T> {
                             let old_inner_size = PhysicalSize::new(width, height);
                             let mut new_inner_size = PhysicalSize::new(new_width, new_height);
 
-                            // Temporarily unlock shared state to prevent deadlock
-                            MutexGuard::unlocked(&mut shared_state_lock, || {
-                                callback(Event::WindowEvent {
-                                    window_id,
-                                    event: WindowEvent::ScaleFactorChanged {
-                                        scale_factor: new_scale_factor,
-                                        new_inner_size: &mut new_inner_size,
-                                    },
-                                });
+                            // Unlock shared state to prevent deadlock in callback below
+                            drop(shared_state_lock);
+
+                            callback(Event::WindowEvent {
+                                window_id,
+                                event: WindowEvent::ScaleFactorChanged {
+                                    scale_factor: new_scale_factor,
+                                    new_inner_size: &mut new_inner_size,
+                                },
                             });
 
                             if new_inner_size != old_inner_size {
@@ -457,7 +461,8 @@ impl<T: 'static> EventProcessor<T> {
                                     new_inner_size.width,
                                     new_inner_size.height,
                                 );
-                                shared_state_lock.dpi_adjusted = Some(new_inner_size.into());
+                                window.shared_state_lock().dpi_adjusted =
+                                    Some(new_inner_size.into());
                                 // if the DPI factor changed, force a resize event to ensure the logical
                                 // size is computed with the right DPI factor
                                 resized = true;
@@ -465,6 +470,8 @@ impl<T: 'static> EventProcessor<T> {
                         }
                     }
 
+                    let mut shared_state_lock = window.shared_state_lock();
+
                     // This is a hack to ensure that the DPI adjusted resize is actually applied on all WMs. KWin
                     // doesn't need this, but Xfwm does. The hack should not be run on other WMs, since tiling
                     // WMs constrain the window size, making the resize fail. This would cause an endless stream of
@@ -478,10 +485,10 @@ impl<T: 'static> EventProcessor<T> {
                         }
                     }
 
-                    if resized {
-                        // Drop the shared state lock to prevent deadlock
-                        drop(shared_state_lock);
+                    // Unlock shared state to prevent deadlock in callback below
+                    drop(shared_state_lock);
 
+                    if resized {
                         callback(Event::WindowEvent {
                             window_id,
                             event: WindowEvent::Resized(new_inner_size.into()),
@@ -504,7 +511,24 @@ impl<T: 'static> EventProcessor<T> {
                     window.invalidate_cached_frame_extents();
                 });
             }
+            ffi::MapNotify => {
+                let xev: &ffi::XMapEvent = xev.as_ref();
+                let window = xev.window;
+                let window_id = mkwid(window);
 
+                // XXX re-issue the focus state when mapping the window.
+                //
+                // The purpose of it is to deliver initial focused state of the newly created
+                // window, given that we can't rely on `CreateNotify`, due to it being not
+                // sent.
+                let focus = self
+                    .with_window(window, |window| window.has_focus())
+                    .unwrap_or_default();
+                callback(Event::WindowEvent {
+                    window_id,
+                    event: WindowEvent::Focused(focus),
+                });
+            }
             ffi::DestroyNotify => {
                 let xev: &ffi::XDestroyWindowEvent = xev.as_ref();
 
@@ -513,7 +537,7 @@ impl<T: 'static> EventProcessor<T> {
 
                 // In the event that the window's been destroyed without being dropped first, we
                 // cleanup again here.
-                wt.windows.borrow_mut().remove(&WindowId(window as u64));
+                wt.windows.borrow_mut().remove(&WindowId(window as _));
 
                 // Since all XIM stuff needs to happen from the same thread, we destroy the input
                 // context here instead of when dropping the window.
@@ -753,7 +777,7 @@ impl<T: 'static> EventProcessor<T> {
                         update_modifiers!(modifiers, None);
 
                         let cursor_moved = self.with_window(xev.event, |window| {
-                            let mut shared_state_lock = window.shared_state.lock();
+                            let mut shared_state_lock = window.shared_state_lock();
                             util::maybe_change(&mut shared_state_lock.cursor_pos, new_cursor_pos)
                         });
                         if cursor_moved == Some(true) {
@@ -923,6 +947,10 @@ impl<T: 'static> EventProcessor<T> {
                             let window_id = mkwid(xev.event);
                             let position = PhysicalPosition::new(xev.event_x, xev.event_y);
 
+                            if let Some(window) = self.with_window(xev.event, Arc::clone) {
+                                window.shared_state_lock().has_focus = true;
+                            }
+
                             callback(Event::WindowEvent {
                                 window_id,
                                 event: Focused(true),
@@ -995,6 +1023,10 @@ impl<T: 'static> EventProcessor<T> {
                                 event: WindowEvent::ModifiersChanged(ModifiersState::empty()),
                             });
 
+                            if let Some(window) = self.with_window(xev.event, Arc::clone) {
+                                window.shared_state_lock().has_focus = false;
+                            }
+
                             callback(Event::WindowEvent {
                                 window_id,
                                 event: Focused(false),
@@ -1014,8 +1046,7 @@ impl<T: 'static> EventProcessor<T> {
                         if self.window_exists(xev.event) {
                             let id = xev.detail as u64;
                             let modifiers = self.device_mod_state.modifiers();
-                            let location =
-                                PhysicalPosition::new(xev.event_x as f64, xev.event_y as f64);
+                            let location = PhysicalPosition::new(xev.event_x, xev.event_y);
 
                             // Mouse cursor position changes when touch events are received.
                             // Only the first concurrently active touch ID moves the mouse cursor.
@@ -1221,7 +1252,7 @@ impl<T: 'static> EventProcessor<T> {
                                                 new_monitor.scale_factor,
                                                 width,
                                                 height,
-                                                &*window.shared_state.lock(),
+                                                &window.shared_state_lock(),
                                             );
 
                                             let window_id = crate::window::WindowId(*window_id);
